from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class PressureEffect:
    r"""The PressureEffect object defines equivalent pressure stress driven mass diffusion.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].diffusivity.pressureEffect
            import odbMaterial
            session.odbs[name].materials[name].diffusivity.pressureEffect

        The table data for this object are:

        - Pressure stress factor, :math:`\kappa_p`.
        - Concentration.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - KAPPA
    """

    @abaqus_method_doc
    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a PressureEffect object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].diffusivity.PressureEffect
                session.odbs[name].materials[name].diffusivity.PressureEffect

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        PressureEffect
            A PressureEffect object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the PressureEffect object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

import inspect


class _OptionsBase:
    """This is a base class for all options classes."""

    @classmethod
    def get_user_attributes(cls) -> list[str]:
        """Returns a list of the user attributes of the class.

        Returns
        -------
        list
            The list of user attributes of the class.
        """
        boring = dir(type("dummy", (object,), {}))
        return [
            key
            for key, value in inspect.getmembers(cls)
            if key not in boring and not key.startswith("_") and not callable(value)
        ]

    def setValues(self, **kwargs):
        """This method modifies the _OptionsBase object.

        Parameters
        ----------
        kwargs
            The keyword arguments.
        """
        for key, value in kwargs.items():
            setattr(self, key, value)


class _CopyOptionsBase(_OptionsBase):
    """This is a base class for options classes where the first argument is the options object used to copy
    options in the setValues method."""

    def setValues(self, options, **kwargs):
        """This method modifies the CopyOptions object.

        Parameters
        ----------
        options : CopyOptions
            An object from which values are to be copied. If other arguments are also supplied to setValues,
            they will override the values in **options**. The default value is None.
        kwargs
            The keyword arguments.
        """
        if options is not None:
            for key in self.get_user_attributes():
                setattr(self, key, getattr(options, key))

        for key, value in kwargs:
            setattr(self, key, value)



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    COMPUTED,
    DEFAULT,
    OFF,
    ON,
    SOLVER_DEFAULT,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Constraint import Constraint


@abaqus_class_doc
class Tie(Constraint):
    """The Tie object defines two surfaces to be tied together for the duration of a simulation. The Tie object
    is derived from the ConstrainedSketchConstraint object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].constraints[name]

        The corresponding analysis keywords are:

        - TIE
    """

    #: A Boolean specifying whether the constraint is suppressed or not. The default value is
    #: OFF.
    suppressed: Boolean = OFF

    #: A String specifying the constraint repository key.
    name: str

    #: A Region object specifying the name of the master surface.
    master: Region

    #: A Region object specifying the name of the slave surface.
    slave: Region

    #: A Boolean specifying whether initial positions of tied slave nodes are adjusted to
    #: lie on the master surface. The default value is ON.
    adjust: Boolean = ON

    #: A SymbolicConstant specifying the method used to determine the position tolerance.
    #: Possible values are COMPUTED and SPECIFIED. The default value is COMPUTED.
    positionToleranceMethod: SymbolicConstant = COMPUTED

    #: A Float specifying the position tolerance. The **positionTolerance** argument applies only
    #: when **positionToleranceMethod** = SPECIFIED. The default value is 0.0.
    positionTolerance: float = 0

    #: A Boolean specifying whether rotation degrees of freedom should be tied. The default
    #: value is ON.
    tieRotations: Boolean = ON

    #: A SymbolicConstant specifying the method used to determine the constraint ratio.
    #: Possible values are DEFAULT and SPECIFIED. The default value is DEFAULT.
    constraintRatioMethod: SymbolicConstant = DEFAULT

    #: A Float specifying the fractional distance between the master reference surface and the
    #: slave node at which the translational constraint should act. The **constraintRatio**
    #: argument applies only when **constraintRatioMethod** = SPECIFIED. The default value is 0.0.
    constraintRatio: float = 0

    #: A SymbolicConstant specifying the discretization method. Possible values are
    #: SOLVER_DEFAULT, NODE_TO_SURFACE, and SURFACE_TO_SURFACE. The default value is
    #: SOLVER_DEFAULT.
    constraintEnforcement: SymbolicConstant = SOLVER_DEFAULT

    #: A Boolean specifying whether shell element thickness is considered. The default value is
    #: ON.
    thickness: Boolean = ON

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        master: Region,
        slave: Region,
        adjust: Boolean = ON,
        positionToleranceMethod: Literal[C.COMPUTED, C.SPECIFIED] = COMPUTED,
        positionTolerance: float = 0,
        tieRotations: Boolean = ON,
        constraintRatioMethod: Literal[C.DEFAULT, C.SPECIFIED] = DEFAULT,
        constraintRatio: float = 0,
        constraintEnforcement: Literal[C.NODE_TO_SURFACE, C.SOLVER_DEFAULT, C.SURFACE_TO_SURFACE] = SOLVER_DEFAULT,
        thickness: Boolean = ON,
    ):
        """This method creates a Tie object.

        .. note::
            This function can be accessed by::

                mdb.models[name].Tie

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        master
            A Region object specifying the name of the master surface.
        slave
            A Region object specifying the name of the slave surface.
        adjust
            A Boolean specifying whether initial positions of tied slave nodes are adjusted to
            lie on the master surface. The default value is ON.
        positionToleranceMethod
            A SymbolicConstant specifying the method used to determine the position tolerance.
            Possible values are COMPUTED and SPECIFIED. The default value is COMPUTED.
        positionTolerance
            A Float specifying the position tolerance. The **positionTolerance** argument applies only
            when **positionToleranceMethod** = SPECIFIED. The default value is 0.0.
        tieRotations
            A Boolean specifying whether rotation degrees of freedom should be tied. The default
            value is ON.
        constraintRatioMethod
            A SymbolicConstant specifying the method used to determine the constraint ratio.
            Possible values are DEFAULT and SPECIFIED. The default value is DEFAULT.
        constraintRatio
            A Float specifying the fractional distance between the master reference surface and the
            slave node at which the translational constraint should act. The **constraintRatio**
            argument applies only when **constraintRatioMethod** = SPECIFIED. The default value is 0.0.
        constraintEnforcement
            A SymbolicConstant specifying the discretization method. Possible values are
            SOLVER_DEFAULT, NODE_TO_SURFACE, and SURFACE_TO_SURFACE. The default value is
            SOLVER_DEFAULT.
        thickness
            A Boolean specifying whether shell element thickness is considered. The default value is
            ON.

        Returns
        -------
        Tie
            A Tie object.
        """
        super().__init__()

    @abaqus_method_doc
    def swapSurfaces(self):
        """This method switches the master and slave surfaces of a tied constraint.

        This command is valid only during the step in which the interaction is created.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        adjust: Boolean = ON,
        positionToleranceMethod: Literal[C.COMPUTED, C.SPECIFIED] = COMPUTED,
        positionTolerance: float = 0,
        tieRotations: Boolean = ON,
        constraintRatioMethod: Literal[C.DEFAULT, C.SPECIFIED] = DEFAULT,
        constraintRatio: float = 0,
        constraintEnforcement: Literal[C.NODE_TO_SURFACE, C.SOLVER_DEFAULT, C.SURFACE_TO_SURFACE] = SOLVER_DEFAULT,
        thickness: Boolean = ON,
    ):
        """This method modifies the Tie object.

        Parameters
        ----------
        adjust
            A Boolean specifying whether initial positions of tied slave nodes are adjusted to
            lie on the master surface. The default value is ON.
        positionToleranceMethod
            A SymbolicConstant specifying the method used to determine the position tolerance.
            Possible values are COMPUTED and SPECIFIED. The default value is COMPUTED.
        positionTolerance
            A Float specifying the position tolerance. The **positionTolerance** argument applies only
            when **positionToleranceMethod** = SPECIFIED. The default value is 0.0.
        tieRotations
            A Boolean specifying whether rotation degrees of freedom should be tied. The default
            value is ON.
        constraintRatioMethod
            A SymbolicConstant specifying the method used to determine the constraint ratio.
            Possible values are DEFAULT and SPECIFIED. The default value is DEFAULT.
        constraintRatio
            A Float specifying the fractional distance between the master reference surface and the
            slave node at which the translational constraint should act. The **constraintRatio**
            argument applies only when **constraintRatioMethod** = SPECIFIED. The default value is 0.0.
        constraintEnforcement
            A SymbolicConstant specifying the discretization method. Possible values are
            SOLVER_DEFAULT, NODE_TO_SURFACE, and SURFACE_TO_SURFACE. The default value is
            SOLVER_DEFAULT.
        thickness
            A Boolean specifying whether shell element thickness is considered. The default value is
            ON.
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    KINEMATIC_HARDENING,
    MAGNITUDE,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .PredefinedField import PredefinedField


@abaqus_class_doc
class KinematicHardening(PredefinedField):
    """The KinematicHardening object stores the data for initial equivalent Plastic strains and, if relevant,
    the initial backstress tensor. The KinematicHardening object is derived from the PredefinedField object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name]

        The corresponding analysis keywords are:

        - INITIAL CONDITIONS
    """

    #: A String specifying the name of the AnalyticalField object associated with this
    #: predefined field. The **field** argument applies only when
    #: **distributionType** = ANALYTICAL_FIELD. The default value is an empty string.
    field: str = ""

    #: A String specifying the repository key.
    name: str

    #: A Region object specifying the region to which the predefined field is applied.
    region: Region

    #: An Int specifying the number of backstresses. The default value is 1.
    numBackStress: int = 1

    #: A sequence of Floats specifying the initial equivalent Plastic strain.
    equivPlasticStrain: tuple[float, ...] = ()

    #: A sequence of sequences of Floats specifying the initial backstress tensor for kinematic
    #: hardening models. The default value is an empty sequence.
    backStress: tuple[tuple[float, ...], ...] = ()

    #: A sequence of Ints specifying section point numbers. This argument is valid only when
    #: **definition** = SECTION_PTS.
    sectPtNum: tuple[int, ...] = ()

    #: A SymbolicConstant specifying different types of kinematic hardening. Possible values
    #: are KINEMATIC_HARDENING, CRUSHABLE_FOAM, REBAR, SECTION_PTS, and USER_DEFINED. The
    #: default value is KINEMATIC_HARDENING.
    definition: SymbolicConstant = KINEMATIC_HARDENING

    #: A sequence of Strings specifying rebar layer names. This argument is valid only when
    #: **definition** = REBAR.
    rebarLayerNames: tuple[str, ...] = ()

    #: A SymbolicConstant specifying whether the load is uniform. Possible values are MAGNITUDE
    #: and ANALYTICAL_FIELD. The default value is MAGNITUDE.
    distributionType: SymbolicConstant = MAGNITUDE

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        numBackStress: int = 1,
        equivPlasticStrain: tuple = (),
        backStress: tuple = (),
        sectPtNum: tuple = (),
        definition: Literal[
            C.CRUSHABLE_FOAM, C.KINEMATIC_HARDENING, C.REBAR, C.USER_DEFINED, C.SECTION_PTS
        ] = KINEMATIC_HARDENING,
        rebarLayerNames: tuple = (),
        distributionType: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
    ):
        """This method creates a KinematicHardening object.

        .. note::
            This function can be accessed by::

                mdb.models[name].KinematicHardening

        Parameters
        ----------
        name
            A String specifying the repository key.
        region
            A Region object specifying the region to which the predefined field is applied.
        numBackStress
            An Int specifying the number of backstresses. The default value is 1.
        equivPlasticStrain
            A sequence of Floats specifying the initial equivalent Plastic strain.
        backStress
            A sequence of sequences of Floats specifying the initial backstress tensor for kinematic
            hardening models. The default value is an empty sequence.
        sectPtNum
            A sequence of Ints specifying section point numbers. This argument is valid only when
            **definition** = SECTION_PTS.
        definition
            A SymbolicConstant specifying different types of kinematic hardening. Possible values
            are KINEMATIC_HARDENING, CRUSHABLE_FOAM, REBAR, SECTION_PTS, and USER_DEFINED. The
            default value is KINEMATIC_HARDENING.
        rebarLayerNames
            A sequence of Strings specifying rebar layer names. This argument is valid only when
            **definition** = REBAR.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are MAGNITUDE
            and ANALYTICAL_FIELD. The default value is MAGNITUDE.

        Returns
        -------
        KinematicHardening
            A KinematicHardening object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        numBackStress: int = 1,
        equivPlasticStrain: tuple = (),
        backStress: tuple = (),
        sectPtNum: tuple = (),
        definition: Literal[
            C.CRUSHABLE_FOAM, C.KINEMATIC_HARDENING, C.REBAR, C.USER_DEFINED, C.SECTION_PTS
        ] = KINEMATIC_HARDENING,
        rebarLayerNames: tuple = (),
        distributionType: Literal[C.MAGNITUDE, C.ANALYTICAL_FIELD] = MAGNITUDE,
    ):
        """This method modifies the KinematicHardening object.

        Parameters
        ----------
        numBackStress
            An Int specifying the number of backstresses. The default value is 1.
        equivPlasticStrain
            A sequence of Floats specifying the initial equivalent Plastic strain.
        backStress
            A sequence of sequences of Floats specifying the initial backstress tensor for kinematic
            hardening models. The default value is an empty sequence.
        sectPtNum
            A sequence of Ints specifying section point numbers. This argument is valid only when
            **definition** = SECTION_PTS.
        definition
            A SymbolicConstant specifying different types of kinematic hardening. Possible values
            are KINEMATIC_HARDENING, CRUSHABLE_FOAM, REBAR, SECTION_PTS, and USER_DEFINED. The
            default value is KINEMATIC_HARDENING.
        rebarLayerNames
            A sequence of Strings specifying rebar layer names. This argument is valid only when
            **definition** = REBAR.
        distributionType
            A SymbolicConstant specifying whether the load is uniform. Possible values are MAGNITUDE
            and ANALYTICAL_FIELD. The default value is MAGNITUDE.
        """
        ...



"""symbolicConstants.py.

Replacement for Abaqus symbolicConstants module.

Defines new style class SymbolicConstant (and SymbolicConstantType) and AbaqusBoolean (and
AbaqusBooleanType).

Defines TRUE, FALSE as Python Boolean objects (True, False). Also defines an AbaqusBoolean, used for ON, and
OFF.

Each SymbolicConstant object is a singleton. The id of each SymbolicConstant is set in the constructor.

AbaqusBoolean is derived from int; there are 2 instances, ON and OFF. The AbaqusBoolean objects are
singletons, they are also in the same cache as the SymbolicConstant objects.

The following applies to Abaqus/CAE only:

It is important that id's of Python SymbolicConstant objects are the same as the id's of C++
omu_SymbolicConstantData objects in omu_SymbolicConstantTable. It is prefereable that id's of
SymbolicConstants are the same in Abaqus/CAE kernel and GUI. To ensure that the id's of the
SymbolicConstants in abaqusConstants are the same in GUI and kernel of CAE, the _symConstCacheInit module is
imported in SMAPybPyoinitialize. This creates all the SymbolicConstant objects in the abaqusConstants
module, and makes this module aware of the cache by adding the _addSymConstToTable function to this module.
Outside Abaqus/CAE, we do not have the same requirement.
"""

from __future__ import annotations

from abaqusConstants import *



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    OFF,
    OPT_DATASAVE_SPECIFY_CYCLE,
    OPT_EXTRACT_SMOOTH_ABAQUS_INPUT_FILE,
    Boolean,
)


@abaqus_class_doc
class OptimizationProcess:
    """The OptimizationProcess object defines a process to perform an optimization of a model defined using an
    optimization task.

    .. note::
        This object can be accessed by::

            import job
            mdb.optimizationProcesses[name]
    """

    #: A String specifying name of the optimization process.
    name: str

    #: A String specifying name of the model to be used for the optimization process.
    model: str

    #: A String specifying name of the optimization task to be used for the optimization
    #: process.
    task: str

    #: A String specifying name of the job to be used as the prototype for all analysis jobs
    #: run by the optimization process.
    prototypeJob: str

    #: A String specifying a description of the optimization process.
    description: str = ""

    #: An Int specifying the maximum number of allowed design cycles for the optimization
    #: process. The default value is 15.
    maxDesignCycle: int = 15

    #: An Enum specifying whether Abaqus should save every iteration file in the optimization
    #: process or a selection of iteration files saved at a user-specified frequency. If you
    #: set **dataSaveFrequency** = OPT_DATASAVE_EVERY_CYCLE, Abaqus saves every iteration file; if
    #: you set **dataSaveFrequency** = OPT_DATASAVE_SPECIFY_CYCLE, Abaqus saves iteration files
    #: according to the frequency defined by the **saveEvery** parameter. The default value is
    #: OPT_DATASAVE_SPECIFY_CYCLE.
    dataSaveFrequency: str = OPT_DATASAVE_SPECIFY_CYCLE

    #: A Boolean specifying whether the initial cycle should be saved when **dataSaveFrequency**
    #: is OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
    saveInitial: Boolean = True

    #: A Boolean specifying whether the first cycle should be saved when **dataSaveFrequency** is
    #: OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
    saveFirst: Boolean = True

    #: A Boolean specifying whether the last cycle should be saved when **dataSaveFrequency** is
    #: OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
    saveLast: Boolean = True

    #: An Int specifying every nth cycle iterations to be saved when **dataSaveFrequency** is
    #: OPT_DATASAVE_SPECIFY_CYCLE. Abaqus saves file iterations for every nth iteration after
    #: iteration 1; if you set **saveEvery** = 3, Abaqus saves file iterations for cycles 1, 4, 7,
    #: and so on. The default value is None.
    saveEvery: int | None = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        model: str,
        task: str,
        prototypeJob: str,
        description: str = "",
        maxDesignCycle: int = 15,
        dataSaveFrequency: str = OPT_DATASAVE_SPECIFY_CYCLE,
        saveInitial: Boolean = True,
        saveFirst: Boolean = True,
        saveLast: Boolean = True,
        saveEvery: int | None = None,
    ):
        """This method creates an OptimizationProcess object.

        .. note::
            This function can be accessed by::

                mdb.OptimizationProcess

        Parameters
        ----------
        name
            A String specifying name of the optimization process.
        model
            A String specifying name of the model to be used for the optimization process.
        task
            A String specifying name of the optimization task to be used for the optimization
            process.
        prototypeJob
            A String specifying name of the job to be used as the prototype for all analysis jobs
            run by the optimization process.
        description
            A String specifying a description of the optimization process.
        maxDesignCycle
            An Int specifying the maximum number of allowed design cycles for the optimization
            process. The default value is 15.
        dataSaveFrequency
            An Enum specifying whether Abaqus should save every iteration file in the optimization
            process or a selection of iteration files saved at a user-specified frequency. If you
            set **dataSaveFrequency** = OPT_DATASAVE_EVERY_CYCLE, Abaqus saves every iteration file; if
            you set **dataSaveFrequency** = OPT_DATASAVE_SPECIFY_CYCLE, Abaqus saves iteration files
            according to the frequency defined by the **saveEvery** parameter. The default value is
            OPT_DATASAVE_SPECIFY_CYCLE.
        saveInitial
            A Boolean specifying whether the initial cycle should be saved when **dataSaveFrequency**
            is OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveFirst
            A Boolean specifying whether the first cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveLast
            A Boolean specifying whether the last cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveEvery
            An Int specifying every nth cycle iterations to be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. Abaqus saves file iterations for every nth iteration after
            iteration 1; if you set **saveEvery** = 3, Abaqus saves file iterations for cycles 1, 4, 7,
            and so on. The default value is None.

        Returns
        -------
        OptimizationProcess
            An OptimizationProcess object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def writeParAndInputFiles(self):
        """This method allows you to write par and input files for an optimization task."""
        ...

    @abaqus_method_doc
    def submit(self, validate: Boolean = False):
        """This method submits an optimization process.

        Parameters
        ----------
        validate
            A Boolean specifying whether Abaqus should perform the validation of the optimization
            process only. The default value is False.
        """
        ...

    @abaqus_method_doc
    def waitForCompletion(self):
        """This method interrupts the execution of the script until the end of all the analyses.

        If you call the waitForCompletion method and the **status** member is neither SUBMITTED nor RUNNING,
        Abaqus assumes the analysis has either completed or aborted and returns immediately.
        """
        ...

    @abaqus_method_doc
    def extract(
        self,
        outputFileName: str,
        designCycle: str,
        isoValue: float = 0,
        smoothCycles: int = 5,
        reductionPercent: float = 0,
        reductionAngle: str = "",
        targetVolume: float = 0,
        extractFormat: tuple = OPT_EXTRACT_SMOOTH_ABAQUS_INPUT_FILE,
        resultFiltering: str = OFF,
        instances: str = "",
    ):
        """This method extracts a surface mesh from the optimized model.

        Parameters
        ----------
        outputFileName
            Name of the output file for the extracted surface mesh.
        designCycle
            The design cycle number for which the surface mesh should be extracted.
        isoValue
            Value used to determine the positions on the element edges where the new nodes are
            created. Value between 0 and 1. The default value is 0.3.
        smoothCycles
            Number of smoothing cycles; if set to 0, no smoothing is performed. The default value is
            5.
        reductionPercent
            Defines the percent of faces that should be removed during the data reduction. If set to
            0, no data reduction occurs. If set to 100, the data reduction stops when no faces can
            be removed (that is checked using reductionAngle parameter). Value between 0 and 100.
            The default value is 0.
        reductionAngle
            Defines the maximal angle between adjacent faces at a node such that the node may be
            removed during the data reduction. Value in degrees between 0 and 90. The default value
            is15.
        targetVolume
            Defines the target volume that is to be achieved iteratively by varying the isovalue.
            Value between 0 and 1. The default value is 0.
        extractFormat
            Tuple for the types of format of the output. Values are (OPT_EXTRACT_SMOOTH_ABAQUS_INPUT_FILE,
            OPT_EXTRACT_SMOOTH_STL). Default: OPT_EXTRACT_SMOOTH_ABAQUS_INPUT_FILE
        resultFiltering
            Possible string values are OFF or MODERATE or FULL. Defines if the element material
            values are to be filtered (averaged locally) before the isocut, and to what extent. The
            default value is OFF.
        instances
            Defines a list of names of part instances to be used for surface extraction. One file
            (according to extractFormat) is created for each part instance. If the argument is not
            specified, no part instance is selected and surface is generated for the whole model.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        description: str = "",
        maxDesignCycle: int = 15,
        dataSaveFrequency: str = OPT_DATASAVE_SPECIFY_CYCLE,
        saveInitial: Boolean = True,
        saveFirst: Boolean = True,
        saveLast: Boolean = True,
        saveEvery: int | None = None,
    ):
        """This method modifies the OptimizationProcess object.

        Parameters
        ----------
        description
            A String specifying a description of the optimization process.
        maxDesignCycle
            An Int specifying the maximum number of allowed design cycles for the optimization
            process. The default value is 15.
        dataSaveFrequency
            An Enum specifying whether Abaqus should save every iteration file in the optimization
            process or a selection of iteration files saved at a user-specified frequency. If you
            set **dataSaveFrequency** = OPT_DATASAVE_EVERY_CYCLE, Abaqus saves every iteration file; if
            you set **dataSaveFrequency** = OPT_DATASAVE_SPECIFY_CYCLE, Abaqus saves iteration files
            according to the frequency defined by the **saveEvery** parameter. The default value is
            OPT_DATASAVE_SPECIFY_CYCLE.
        saveInitial
            A Boolean specifying whether the initial cycle should be saved when **dataSaveFrequency**
            is OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveFirst
            A Boolean specifying whether the first cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveLast
            A Boolean specifying whether the last cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveEvery
            An Int specifying every nth cycle iterations to be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. Abaqus saves file iterations for every nth iteration after
            iteration 1; if you set **saveEvery** = 3, Abaqus saves file iterations for cycles 1, 4, 7,
            and so on. The default value is None.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Session.SessionBase import SessionBase
from ..UtilityAndView.abaqusConstants import OFF, Boolean
from .Odb import Odb
from .ScratchOdb import ScratchOdb


@abaqus_class_doc
class OdbSession(SessionBase):
    @abaqus_method_doc
    def ScratchOdb(self, odb: Odb) -> ScratchOdb:
        """This method creates a new ScratchOdb object.

        .. note::
            This function can be accessed by::

                session.ScratchOdb

        Parameters
        ----------
        odb
            An Odb object specifying the output database with which to associate.

        Returns
        -------
        ScratchOdb
            A ScratchOdb object.
        """
        self.scratchOdbs["odb"] = scratchOdb = ScratchOdb(odb)
        return scratchOdb

    # The following method was originally in the ``OdbCommands`` page documentation
    # But it accessed only by ``session`` object.
    @abaqus_method_doc
    def openOdb(self, name: str, path: str = "", readOnly: Boolean = OFF) -> Odb:
        """This method opens an existing output database (`.odb`) file and creates a new Odb object. This method
        is accessed only via the session object inside Abaqus/CAE and adds the new Odb object to the
        session.odbs repository. This method allows you to open multiple output databases at the same time and
        to use the repository key to specify a particular output database. For example::

            import visualization
            session.openOdb(name='myOdb', path='stress.odb', readOnly=True)

        Parameters
        ----------
        name
            A String specifying the repository key. If the ``name`` is not the same as the ``path`` to the
            output database (`.odb`) file, the ``path`` must be specified as well. Additionally, to
            support backwards compatibility of the interface, if the ``name`` parameter is omitted,
            the ``path`` and ``name`` will be presumed to be the same.
        path
            A String specifying the path to an existing output database (`.odb`) file.
        readOnly
            A Boolean specifying whether the file will permit only read access or both read and
            write access. The initial value is TRUE when the output database file is opened from
            Abaqus/CAE, indicating that only read access will be permitted.

        Returns
        -------
        Odb
            An Odb object.

        Raises
        ------
        OdbError
            The database is from a previous release of Abaqus, If the output database was generated by a previous release of Abaqus and needs
            upgrading, Run  `abaqus upgrade -job <newFilename> -odb <oldFileName>` to upgrade it.
        OdbError
            Abaqus installation must be upgraded before this output database can be opened, If the output database was generated by a newer release of Abaqus, and the
            installation of Abaqus needs upgrading.
        AbaqusError: Cannot open file <filename>
            If the file is not a valid database.
        """
        self.odbs[name] = odb = Odb(name)
        return odb

    @abaqus_method_doc
    def upgradeOdb(self, existingOdbPath: str, upgradedOdbPath: str):
        """This method upgrades an existing Odb object to the current release and writes the upgraded version of the
        Odb object to a file. In addition, Abaqus/CAE writes information about the status of the upgrade to a log
        (.log) file. You can access this method using either of the following techniques:

        - From a script running outside Abaqus/CAE. For example::

            import odbAccess
            odbAccess.upgradeOdb(existingOdbPath='oldOdb', upgradedOdbPath='upgradedOdb')

        - From the session object in Abaqus/CAE. For example::

            import visualization
            session.upgradeOdb(existingOdbPath='oldOdb', upgradedOdbPath='upgradedOdb')

        Parameters
        ----------
        existingOdbPath
            An String specifying the path to the file containing the output database to be upgraded.
        upgradedOdbPath
            An String specifying the path to the file that will contain the upgraded output
            database.

        Raises
        ------
        OdbError: cannot convert database
            If the output database upgrade fails.
        """
        ...



from __future__ import annotations

from .FXObject import FXObject


class AFXMode(FXObject):
    """This class is the base class for modes."""

    #: Activates the mode.
    ID_ACTIVATE: int = hash("ID_ACTIVATE")

    #: Commits the mode.
    ID_COMMIT: int = hash("ID_COMMIT")

    #: Cancels the mode.
    ID_CANCEL: int = hash("ID_CANCEL")

    #: Deactivates the mode.
    ID_DEACTIVATE: int = hash("ID_DEACTIVATE")

    #: Gets the next step/dialog box.
    ID_GET_NEXT: int = hash("ID_GET_NEXT")

    #: Resumes the mode.
    ID_RESUME: int = hash("ID_RESUME")

    #: Sets defaults.
    ID_SET_DEFAULTS: int = hash("ID_SET_DEFAULTS")

    #: Suspends the mode.
    ID_SUSPEND: int = hash("ID_SUSPEND")

    #: Indicates that a command is activated.
    ID_CMD_ACTIVATED: int = hash("ID_CMD_ACTIVATED")

    #: Indicates that a command is deactivated.
    ID_CMD_DEACTIVATED: int = hash("ID_CMD_DEACTIVATED")

    #: Indicates that a command is modified.
    ID_CMD_MODIFIED: int = hash("ID_CMD_MODIFIED")

    def __init__(self):
        """Constructor."""

    def getCommand(self, index: int):
        """Returns the command at the given index (returns 0 if the index is out-of-bounds).

        Parameters
        ----------
        index : int
            Command index (0-based).
        """

    def getNumCommands(self):
        """Returns the number of commands associated with the mode."""

    def isActive(self):
        """Returns True if the mode is active."""



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    DEFAULT,
    DEFAULT_LIMIT,
    LAST_INCREMENT,
    MODEL,
    OFF,
    ON,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class RemeshingRule:
    """The RemeshingRule object controls the adaptive remeshing resizing and the error indicators written to the
    output database for a specified region of the model.

    .. note::
        This object can be accessed by::

            import mesh
            mdb.models[name].remeshingRules[name]
    """

    #: A Boolean specifying whether the remeshing rule is suppressed. Remeshing of the
    #: remeshing rule's region will not occur if you suppress a rule. The default value is OFF.
    suppressed: Boolean = OFF

    #: A String specifying the name of the object.
    name: str

    #: A String specifying the name of the step in which resizing should occur for this rule.
    stepName: str

    #: A sequence of Strings specifying the output request variables that Abaqus will use as
    #: error indicators.
    variables: tuple[str, ...] = ()

    #: A String specifying a descriptive string for this rule. The default value is an empty
    #: string.
    description: str = ""

    #: The SymbolicConstant MODEL or a Region object specifying the region in which Abaqus will
    #: remesh and generate output. The SymbolicConstant MODEL represents the entire applicable
    #: model. The default value is MODEL.
    region: SymbolicConstant = MODEL

    #: A SymbolicConstant specifying the method for calculating the new mesh sizes. The
    #: SymbolicConstant DEFAULT indicates that Abaqus will use the default calculation method
    #: for each individual variable. Possible values are DEFAULT, UNIFORM_ERROR, and
    #: MINIMUM_MAXIMUM. The default value is DEFAULT.
    sizingMethod: SymbolicConstant = DEFAULT

    #: A Float specifying the target error percentage for each variable in the region. A value
    #: of 0.0 indicates that Abaqus will use automated error target reduction for the region.
    #: You use the **errorTarget** argument when **sizingMethod** = UNIFORM_ERROR. The default value
    #: is 0.0.
    errorTarget: float = 0

    #: A Float specifying the target error percentage at the location of the maximum solution
    #: value in the region. A value of 0.0 indicates that Abaqus will use automated error
    #: target reduction for the region. You use the **maxSolutionErrorTarget** argument when
    #: **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
    maxSolutionErrorTarget: float = 0

    #: A Float specifying the target error percentage at the location of the minimum solution
    #: value in the region. A value of 0.0 indicates that Abaqus will use automated error
    #: target reduction for the region. You use the **minSolutionErrorTarget** argument when
    #: **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
    minSolutionErrorTarget: float = 0

    #: An Int specifying an indication of how much Abaqus will bias the mesh toward the
    #: location of the maximum solution value in the region. The higher the value, the more the
    #: mesh will bias towards the location of the maximum solution value. You use the
    #: **meshBias** argument when **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
    meshBias: int = 0

    #: A Float specifying the minimum size of any single element. The default value is 0.0.
    minElementSize: float = 0

    #: A Float specifying the maximum size of any single element. The default value is 0.0.
    maxElementSize: float = 0

    #: A SymbolicConstant specifying the frequency with which the error indicators are saved to
    #: the output database file (.odb). Possible values are LAST_INCREMENT and ALL_INCREMENTS.
    #: The default value is LAST_INCREMENT.
    outputFrequency: SymbolicConstant = LAST_INCREMENT

    #: A Boolean specifying an indication of whether to use a user-supplied minimum element
    #: size or to calculate a characteristic minimum element size. The default value is OFF.
    specifyMinSize: Boolean = OFF

    #: A Boolean specifying an indication of whether to use a user-supplied maximum element
    #: size or to calculate a characteristic maximum element size. The default value is ON.
    specifyMaxSize: Boolean = ON

    #: A SymbolicConstant or an Int specifying an indication of the upper limit on the element
    #: growth from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT and
    #: NOT_ALLOWED. The default value is DEFAULT_LIMIT.
    coarseningFactor: SymbolicConstant = DEFAULT_LIMIT

    #: A SymbolicConstant or an Int specifying an indication of the upper limit on element
    #: shrinkage from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT
    #: and NOT_ALLOWED. The default value is DEFAULT_LIMIT.
    refinementFactor: SymbolicConstant = DEFAULT_LIMIT

    #: None or an Int specifying an approximate limit on the number of elements that will be
    #: created during remeshing. Use None to indicate there is not upper limit. The default
    #: value is None.
    elementCountLimit: int | None = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        stepName: str,
        variables: tuple,
        description: str = "",
        region: Literal[C.MODEL] = MODEL,
        sizingMethod: Literal[C.MINIMUM_MAXIMUM, C.DEFAULT, C.UNIFORM_ERROR] = DEFAULT,
        errorTarget: float = 0,
        maxSolutionErrorTarget: float = 0,
        minSolutionErrorTarget: float = 0,
        meshBias: int = 0,
        minElementSize: float = 0,
        maxElementSize: float = 0,
        outputFrequency: Literal[C.LAST_INCREMENT, C.ALL_INCREMENTS] = LAST_INCREMENT,
        specifyMinSize: Boolean = OFF,
        specifyMaxSize: Boolean = ON,
        coarseningFactor: Literal[C.NOT_ALLOWED, C.DEFAULT_LIMIT] = DEFAULT_LIMIT,
        refinementFactor: Literal[C.NOT_ALLOWED, C.DEFAULT_LIMIT] = DEFAULT_LIMIT,
        elementCountLimit: int | None = None,
    ):
        """This method creates a RemeshingRule object.

        .. note::
            This function can be accessed by::

                mdb.models[name].RemeshingRule

        Parameters
        ----------
        name
            A String specifying the name of the object.
        stepName
            A String specifying the name of the step in which resizing should occur for this rule.
        variables
            A sequence of Strings specifying the output request variables that Abaqus will use as
            error indicators.
        description
            A String specifying a descriptive string for this rule. The default value is an empty
            string.
        region
            The SymbolicConstant MODEL or a Region object specifying the region in which Abaqus will
            remesh and generate output. The SymbolicConstant MODEL represents the entire applicable
            model. The default value is MODEL.
        sizingMethod
            A SymbolicConstant specifying the method for calculating the new mesh sizes. The
            SymbolicConstant DEFAULT indicates that Abaqus will use the default calculation method
            for each individual variable. Possible values are DEFAULT, UNIFORM_ERROR, and
            MINIMUM_MAXIMUM. The default value is DEFAULT.
        errorTarget
            A Float specifying the target error percentage for each variable in the region. A value
            of 0.0 indicates that Abaqus will use automated error target reduction for the region.
            You use the **errorTarget** argument when **sizingMethod** = UNIFORM_ERROR. The default value
            is 0.0.
        maxSolutionErrorTarget
            A Float specifying the target error percentage at the location of the maximum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **maxSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minSolutionErrorTarget
            A Float specifying the target error percentage at the location of the minimum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **minSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        meshBias
            An Int specifying an indication of how much Abaqus will bias the mesh toward the
            location of the maximum solution value in the region. The higher the value, the more the
            mesh will bias towards the location of the maximum solution value. You use the
            **meshBias** argument when **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minElementSize
            A Float specifying the minimum size of any single element. The default value is 0.0.
        maxElementSize
            A Float specifying the maximum size of any single element. The default value is 0.0.
        outputFrequency
            A SymbolicConstant specifying the frequency with which the error indicators are saved to
            the output database file (.odb). Possible values are LAST_INCREMENT and ALL_INCREMENTS.
            The default value is LAST_INCREMENT.
        specifyMinSize
            A Boolean specifying an indication of whether to use a user-supplied minimum element
            size or to calculate a characteristic minimum element size. The default value is OFF.
        specifyMaxSize
            A Boolean specifying an indication of whether to use a user-supplied maximum element
            size or to calculate a characteristic maximum element size. The default value is ON.
        coarseningFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on the element
            growth from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT and
            NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        refinementFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on element
            shrinkage from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT
            and NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        elementCountLimit
            None or an Int specifying an approximate limit on the number of elements that will be
            created during remeshing. Use None to indicate there is not upper limit. The default
            value is None.

        Returns
        -------
        RemeshingRule
            A RemeshingRule object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def resume(self):
        """This method resumes the remeshing rule that was previously suppressed."""
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses the remeshing rule.

        Abaqus will not remesh regions where the rules are suppressed.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        description: str = "",
        region: Literal[C.MODEL] = MODEL,
        sizingMethod: Literal[C.MINIMUM_MAXIMUM, C.DEFAULT, C.UNIFORM_ERROR] = DEFAULT,
        errorTarget: float = 0,
        maxSolutionErrorTarget: float = 0,
        minSolutionErrorTarget: float = 0,
        meshBias: int = 0,
        minElementSize: float = 0,
        maxElementSize: float = 0,
        outputFrequency: Literal[C.LAST_INCREMENT, C.ALL_INCREMENTS] = LAST_INCREMENT,
        specifyMinSize: Boolean = OFF,
        specifyMaxSize: Boolean = ON,
        coarseningFactor: Literal[C.NOT_ALLOWED, C.DEFAULT_LIMIT] = DEFAULT_LIMIT,
        refinementFactor: Literal[C.NOT_ALLOWED, C.DEFAULT_LIMIT] = DEFAULT_LIMIT,
        elementCountLimit: int | None = None,
    ):
        """This method modifies the RemeshingRule object.

        Parameters
        ----------
        description
            A String specifying a descriptive string for this rule. The default value is an empty
            string.
        region
            The SymbolicConstant MODEL or a Region object specifying the region in which Abaqus will
            remesh and generate output. The SymbolicConstant MODEL represents the entire applicable
            model. The default value is MODEL.
        sizingMethod
            A SymbolicConstant specifying the method for calculating the new mesh sizes. The
            SymbolicConstant DEFAULT indicates that Abaqus will use the default calculation method
            for each individual variable. Possible values are DEFAULT, UNIFORM_ERROR, and
            MINIMUM_MAXIMUM. The default value is DEFAULT.
        errorTarget
            A Float specifying the target error percentage for each variable in the region. A value
            of 0.0 indicates that Abaqus will use automated error target reduction for the region.
            You use the **errorTarget** argument when **sizingMethod** = UNIFORM_ERROR. The default value
            is 0.0.
        maxSolutionErrorTarget
            A Float specifying the target error percentage at the location of the maximum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **maxSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minSolutionErrorTarget
            A Float specifying the target error percentage at the location of the minimum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **minSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        meshBias
            An Int specifying an indication of how much Abaqus will bias the mesh toward the
            location of the maximum solution value in the region. The higher the value, the more the
            mesh will bias towards the location of the maximum solution value. You use the
            **meshBias** argument when **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minElementSize
            A Float specifying the minimum size of any single element. The default value is 0.0.
        maxElementSize
            A Float specifying the maximum size of any single element. The default value is 0.0.
        outputFrequency
            A SymbolicConstant specifying the frequency with which the error indicators are saved to
            the output database file (.odb). Possible values are LAST_INCREMENT and ALL_INCREMENTS.
            The default value is LAST_INCREMENT.
        specifyMinSize
            A Boolean specifying an indication of whether to use a user-supplied minimum element
            size or to calculate a characteristic minimum element size. The default value is OFF.
        specifyMaxSize
            A Boolean specifying an indication of whether to use a user-supplied maximum element
            size or to calculate a characteristic maximum element size. The default value is ON.
        coarseningFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on the element
            growth from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT and
            NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        refinementFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on element
            shrinkage from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT
            and NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        elementCountLimit
            None or an Int specifying an approximate limit on the number of elements that will be
            created during remeshing. Use None to indicate there is not upper limit. The default
            value is None.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class InteractionProperty:
    """The InteractionProperty object is the abstract base type for other InteractionProperty objects. The
    InteractionProperty object has no explicit constructor, members, or methods.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]
    """

    ...



from __future__ import annotations

from .AFXBoolKeyword import AFXBoolKeyword
from .AFXDialog import AFXDialog
from .AFXGuiMode import AFXGuiMode
from .AFXTransition import AFXTransition
from .constants import DIALOG_NORMAL
from .FXObject import FXObject
from .FXWindow import FXWindow

#: Perform bailout checks when the Cancel button is clicked.
DATADIALOG_BAILOUT: int = hash("DATADIALOG_BAILOUT")


class AFXDataDialog(AFXDialog):
    """AFXDataDialog."""

    #: Used to update the state.
    ID_UPDATE_STATE: int = hash("ID_UPDATE_STATE")

    def __init__(
        self,
        mode: AFXGuiMode,
        owner: FXWindow,
        title: str,
        actionButtonIds: int = 0,
        opts: int = DIALOG_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Constructor that creates a dialog box that occludes its owner widget.

        Parameters
        ----------
        mode : AFXGuiMode
            Host mode.
        owner : FXWindow
            Owner widget.
        title : str
            Title string.
        actionButtonIds : int
            ID's of action buttons to be created.
        opts : int
            Options and hints.
        x : int
            X coordinate of origin.
        y : int
            Y coordinate of origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        """

    def addTransition(
        self,
        keyword: AFXBoolKeyword,
        op: AFXTransition.Operator,
        value: bool,
        tgt: FXObject,
        sel: int,
        ptr: str = "None",
    ):
        """Adds a finite state transition to the dialog box. When the expression "keyword.getValue() op value"
        evaluates to True, an sel message will be sent to the tgt object.

        Parameters
        ----------
        keyword : AFXBoolKeyword
            Keyword.
        op : FXTransition.Operator
            Operator type.
        value : bool
            Reference value.
        tgt : FXObject
            Message target.
        sel : int
            Message selector.
        ptr : str
            Message data.
        """

    def bailout(self):
        """Performs checks to determine whether it is OK to cancel the dialog box.

        The implementaton of this class always returns True, and the derived class should reimplement this
        method to perform specific checks. Reimplemented from AFXDialog.
        """

    def getMode(self):
        """Returns the dialog box's host mode."""

    def onKeywordError(self, kwd: FXObject):
        """Handles the error that occurs when the given keyword or target contains invalid contents. This method
        will select the contents of the widget that is set for the keyword or target (with
        setWidgetForKeyword()). If no such widget is specified explicitly, it will select the contents of the
        widget that has the keyword or target as its message target.

        Parameters
        ----------
        kwd : FXObject
            Object that contains invalid contents.
        """

    def onTableError(self, tableKwd: FXObject, row: int, col: int):
        """Handles the error that occurs when the given table keyword or target contains an invalid element.
        This method will select the contents of the widget that is set for the element of the keyword or target
        (with setWidgetForKeyword()). If no such widget is specified explicitly, it will select the contents of
        the widget that has the keyword or target as its message target.

        Parameters
        ----------
        tableKwd : FXObject
            Object that contains invalid element.
        row : int
            Row index.
        col : int
            Column index.
        """

    def onTupleError(self, tupleKwd: FXObject, index: int):
        """Handles the error that occurs when the given tuple keyword or target contains an invalid element.
        This method will select the contents of the widget that is set for the element of the keyword or target
        (with setWidgetForKeyword()). If no such widget is specified explicitly, it will select the contents of
        the widget that has the keyword or target as its message target.

        Parameters
        ----------
        tupleKwd : FXObject
            Object that contains invalid element.
        index : int
            Element index.
        """

    def processUpdates(self):
        """Performs state processing during the GUI update cycles.

        This class provides an empty implementation of this method, and the derived class should redefine
        the method if it needs to process state updating.
        """
